
#include	"FTGeometryGlyph.h"
#include	"FTVectoriser.h"

#ifndef CALLBACK
#define CALLBACK
#endif

// NOT MULTITHREAD SAFE!!!
typedef struct {
    GLdouble x;
    GLdouble y;
    GLdouble z;
} Vertex;
static vector<Vertex*> tri;

void CALLBACK ftglError( GLenum errCode)
{
//	const GLubyte* estring;
//	estring = gluErrorString( errCode);
//	fprintf( stderr, "ERROR : %s\n", estring);
//	exit(1);
}

void CALLBACK ftglVertex( void* data)
{
    double* vertex = (double*)data;

    Vertex* v = new Vertex;
    v->x = vertex[0];
    v->y = vertex[1];
    v->z = vertex[2];
    tri.push_back(v);
}


void CALLBACK ftglBegin( GLenum type)
{
    for (int i=0; i < tri.size(); i++) {
        if (tri[i]) {
            delete tri[i];
        }
    }
    tri.clear();
}


void CALLBACK ftglEnd()
{
}


void CALLBACK ftglCombine( GLdouble coords[3], void* vertex_data[4], GLfloat weight[4], void** outData)
{
	GLdouble* vertex = new GLdouble[3];
	
	vertex[0] = coords[0];
	vertex[1] = coords[1];
	vertex[2] = coords[2];

	*outData = vertex;
}


FTGeometryGlyph::FTGeometryGlyph( FT_Glyph glyph)
:	FTGlyph(),
	numPoints(0),
	numContours(0),
	contourLength(0),
	contours(0),
    numTriangles(0),
    triangles(0)
{
	if (ft_glyph_format_outline == glyph->format) {

        FTVectoriser* vectoriser = new FTVectoriser( glyph);
        if (vectoriser) {
            vectoriser->Process();
            numContours = vectoriser->contours();
            contourLength = new int[ numContours];
            if (contourLength) {
                for( int c = 0; c < numContours; ++c) {
                    contourLength[c] = vectoriser->contourSize( c);
                }
	
                numPoints = vectoriser->points();
                contours = new double[ numPoints * 3];
                if (contours) {
                    vectoriser->MakeOutline( contours);
                
                    contourFlag = vectoriser->ContourFlag();
                    advance = glyph->advance.x >> 16;
                }
            }
            delete vectoriser;

            if ( ( numContours >= 1) && ( numPoints >= 3)) {
                Tesselate();
            }
        }
    }

	// discard glyph image (bitmap or not)
	FT_Done_Glyph( glyph); // Why does this have to be HERE
}


void FTGeometryGlyph::Tesselate()
{
	GLUtesselator* tobj = gluNewTess();
	int d = 0;
	
	gluTessCallback( tobj, GLU_TESS_BEGIN, (void (CALLBACK*)())ftglBegin);
	gluTessCallback( tobj, GLU_TESS_VERTEX, (void (CALLBACK*)())ftglVertex);
	gluTessCallback( tobj, GLU_TESS_COMBINE, (void (CALLBACK*)())ftglCombine);
	gluTessCallback( tobj, GLU_TESS_END, ftglEnd);
	gluTessCallback( tobj, GLU_TESS_ERROR, (void (CALLBACK*)())ftglError);
	
    if( contourFlag & ft_outline_even_odd_fill) // ft_outline_reverse_fill
    {
        gluTessProperty( tobj, GLU_TESS_WINDING_RULE, GLU_TESS_WINDING_ODD);
    }
    else
    {
        gluTessProperty( tobj, GLU_TESS_WINDING_RULE, GLU_TESS_WINDING_NONZERO);
    }
		
    gluTessProperty( tobj, GLU_TESS_TOLERANCE, 0);
    gluTessBeginPolygon( tobj, NULL);
		
    for( int c = 0; c < numContours; ++c)
    {
        gluTessBeginContour( tobj);
        for( int p = 0; p < ( contourLength[c]); ++p)
        {
            gluTessVertex( tobj, contours + d, contours + d);
            d += 3;
        }
        gluTessEndContour( tobj);
    }
			
    gluTessEndPolygon( tobj);
		
	gluDeleteTess( tobj);

    // Save the triangles.
    numTriangles = tri.size();
    triangles = new double[numTriangles*3];
    for (int v=0; v < numTriangles; ++v) {
        triangles[v*3]   = tri[v]->x;
        triangles[v*3+1] = tri[v]->y;
        triangles[v*3+2] = tri[v]->z;
        delete tri[v];
        tri[v] = 0;
    }
    tri.clear();
}


FTGeometryGlyph::~FTGeometryGlyph()
{
    if (contours)
        delete [] contours;
    if (contourLength)
        delete [] contourLength;
    if (triangles)
        delete [] triangles;
}


float FTGeometryGlyph::Render( const FT_Vector& pen)
{
	return advance;
}
