#ifndef		__FTGeometryGlyph__
#define		__FTGeometryGlyph__

#include "FTGL.h"

#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_GLYPH_H

#include <vector>

#include "FTGlyph.h"

using namespace std;

/**
 * FTGeometryGlyph is a specialisation of FTGlyph for creating tessellated
 * glyph polygons and outlines.
 * 
 * @see FTGlyphContainer
 *
 */
class FTGL_EXPORT FTGeometryGlyph : public FTGlyph
{
	public:
		/**
		 * Constructor
		 *
		 * @param glyph	The Freetype glyph to be processed
		 */
		FTGeometryGlyph( FT_Glyph glyph);

		/**
		 * Destructor
		 */
		virtual ~FTGeometryGlyph();

		/**
		 * Renders this glyph at the current pen position.
		 *
		 * @param pen	The current pen position.
		 * @return		The advance distance for this glyph.
		 */
		virtual float Render( const FT_Vector& pen);
		
	private:
		/**
		 * Convert the point data into a mesh.
		 *
		 * Uses GLUtesselator to create a mesh
		 */
		void Tesselate();
		
    public:
		/**
		 * The total number of points in the Freetype outline
		 */
		int numPoints;

		/**
		 * The totals number of contours in the Freetype outline
		 */
		int numContours;

		/**
		 * An flag indicating the tesselation rules for this glyph
		 */
		int contourFlag; 

		/**
		 * An array containing the number of points in each outline
		 */
		int* contourLength;

		/**
		 * Pointer to the contour point data
		 */
		double* contours;
		
        /**
         * Number of vertices in the triangles array.
         */
        int numTriangles;

        /**
         * Pointer to the triangle data
         */
        double* triangles;
};


#endif	//	__FTGeometryGlyph__

