// extrude.cc
//
// Based on the logo.C demo of the gltt graphics library by Stephane Rehel.
// 
//   Copyright (c) Gerard Lanois, 2001.
//
//   This program is freely distributable without licensing fees and is
//   provided without guarantee or warrantee expressed or implied. This
//   program is -not- in the public domain. 


#include <stdlib.h>   // exit()
#include <iostream.h> // cerr
#include <math.h>     // cos(), sin()

#ifdef WIN32
#include <windows.h>
#define M_PI (2.0*asin(1.0))
#endif

#include "GL/glut.h"

#include "ExtrudedString.h"

static char* g_font_name = 0;
static const char* g_text= "OpenGL!";
static GLdouble g_extrude_length = 30.0;
static int g_width  = 640;
static int g_height = 9*g_width/16;
static GLdouble g_phi   = 0.0;
static GLdouble g_theta = 0.0;
static GLdouble g_camera_dist = 500.0;
static ExtrudedString* g_es = 0;


#include "FTGLOutlineFont.h"
static FTGLOutlineFont ofont;

static void 
spheric_camera(
   GLdouble center_x, 
   GLdouble center_y, 
   GLdouble center_z,
   GLdouble phi, 
   GLdouble theta, 
   GLdouble radius)
{
  GLdouble x = center_x + cos(phi) * cos(theta) * radius;
  GLdouble y = center_y + sin(phi) * cos(theta) * radius;
  GLdouble z = center_z +            sin(theta) * radius;

  GLdouble vx = -cos(phi) * sin(theta) * radius;
  GLdouble vy = -sin(phi) * sin(theta) * radius;
  GLdouble vz =             cos(theta) * radius;

  glViewport(0, 0, g_width, g_height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(60.0, GLdouble(g_width)/GLdouble(g_height), 10.0, 10000.0);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(x, y, z, center_x, center_y, center_z, vx, vy, vz );
  glRotatef(90.0, 0.0, 0.0, 1.0);
}


static void
my_lighting()
{
   // Set up lighting.
   float light1_ambient[4]  = { 0.2, 0.2, 0.2, 1.0 };
   float light1_diffuse[4]  = { 0.9, 0.9, 0.9, 1.0 };
   float light1_specular[4] = { 0.7, 0.7, 0.7, 1.0 };
   float light1_position[4] = { -1.0, 1.0, 1.0, 0.0 };
   glLightfv(GL_LIGHT1, GL_AMBIENT,  light1_ambient);
   glLightfv(GL_LIGHT1, GL_DIFFUSE,  light1_diffuse);
   glLightfv(GL_LIGHT1, GL_SPECULAR, light1_specular);
   glLightfv(GL_LIGHT1, GL_POSITION, light1_position);
   glEnable(GL_LIGHT1);

   float light2_ambient[4]  = { 0.2, 0.2, 0.2, 1.0 };
   float light2_diffuse[4]  = { 0.9, 0.9, 0.9, 1.0 };
   float light2_specular[4] = { 0.7, 0.7, 0.7, 1.0 };
   float light2_position[4] = { 1.0, -1.0, -1.0, 0.0 };
   glLightfv(GL_LIGHT2, GL_AMBIENT,  light2_ambient);
   glLightfv(GL_LIGHT2, GL_DIFFUSE,  light2_diffuse);
   glLightfv(GL_LIGHT2, GL_SPECULAR, light2_specular);
   glLightfv(GL_LIGHT2, GL_POSITION, light2_position);
   glEnable(GL_LIGHT2);

   float front_emission[4] = { 0.0, 0.0, 0.0, 0.0 };
   float front_ambient[4]  = { 0.2, 0.2, 0.2, 0.0 };
   float front_diffuse[4]  = { 0.95, 0.95, 0.8, 0.0 };
   float front_specular[4] = { 0.6, 0.6, 0.6, 0.0 };
   glMaterialfv(GL_FRONT, GL_EMISSION, front_emission);
   glMaterialfv(GL_FRONT, GL_AMBIENT, front_ambient);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, front_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, front_specular);
   glMaterialf(GL_FRONT, GL_SHININESS, 16.0);
   glColor4fv(front_diffuse);

   glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
   glCullFace(GL_BACK);
   glFrontFace(GL_CCW);
   glEnable(GL_CULL_FACE);
   glColorMaterial(GL_FRONT, GL_DIFFUSE);
   glEnable(GL_COLOR_MATERIAL);

   glEnable(GL_LIGHTING);
   glShadeModel(GL_SMOOTH);
   glEnable(GL_NORMALIZE);
}


static void 
my_display_func()
{
   if (!g_es) {
      g_es = new ExtrudedString(g_font_name, g_text);
   }

   if (g_es) {

      my_lighting();

      // Clear color and depth buffers.
      glClearColor(0.0, 0.0, 0.0, 0.0);
      glEnable(GL_DEPTH_TEST);
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

      // Set up matrices.
      int llx, lly, urx, ury;
      g_es->getBBox(llx, lly, urx, ury);
      spheric_camera(
         -GLdouble(ury-lly)/2.0, 
         GLdouble(urx-llx)/2.0, 
         0.0, 
         g_phi, 
         g_theta+M_PI/2.0, 
         g_camera_dist);

      // Now draw it.
      g_es->draw(g_extrude_length);
      glDisable(GL_LIGHTING);
      glColor3f(1.0, 1.0, 1.0);
      ofont.render("Booga Booga Booga!");
      glutSwapBuffers();
   }
}


static void 
my_reshape_func( 
   int new_width, 
   int new_height)
{
   g_width = new_width;
   g_height= new_height;
}


static void 
my_keyboard_func(
   unsigned char key, 
   int /*x*/, 
   int /*y*/ )
{
  switch (key) {
  case 27: 
     exit(1);
     break;
  case 's':
     g_camera_dist /= 1.2;
     break;
  case 'S':
     g_camera_dist *= 1.2;
     break;
  case 'e':
     g_extrude_length -= 5.0;
     break;
  case 'E':
     g_extrude_length += 5.0;
     break;
  }

  glutPostRedisplay();
}


static void 
my_special_func(
   int key, 
   int /*x*/, 
   int /*y*/ )
{
  const double angle_speed= 10./180.*M_PI;

  switch (key) {
  case GLUT_KEY_UP:
     g_theta += angle_speed;
     break;
  case GLUT_KEY_DOWN:
     g_theta -= angle_speed;
     break;
  case GLUT_KEY_LEFT:
     g_phi -= angle_speed;
     break;
  case GLUT_KEY_RIGHT:
     g_phi += angle_speed;
     break;
  }

  glutPostRedisplay();
}


int 
main(
   int argc, 
   char **argv)
{
   glutInit(&argc, argv);
   if (argc != 3) {
      cerr << argv[0] << ": bad arguments.\nUsage:\n";
      cerr << "\t" << argv[0] << " <font_filename.ttf> \"text\"\n\n";
      cerr << "\t" << "Example:\n\t ";
      cerr << argv[0] << " arial.ttf \"OpenGL\"\n\n";
   }
   else { 
      g_font_name = argv[1];
      g_text = argv[2];

        if (!ofont.Open(g_font_name)) {
            cerr << "ERROR: Unable to open file " << g_font_name << "\n";
        }
        else {
            int point_size = 24;
            if (!ofont.FaceSize(point_size)) {
                cerr << "ERROR: Unable to set font face size " << point_size << "\n";
            }
        }

      glutInitWindowSize(g_width, g_height);
      glutInitDisplayMode( GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE );

      glutCreateWindow("extrude");

      glutDisplayFunc(my_display_func);
      glutReshapeFunc(my_reshape_func);
      glutKeyboardFunc(my_keyboard_func);
      glutSpecialFunc(my_special_func);
      glutDisplayFunc(my_display_func);

      cerr << "Key Usage:\n";
      cerr << "s(zoom in)  S(zoom out)\n";
      cerr << "left(dec camera phi) right(inc camera theta)\n";
      cerr << "up(inc camera theta) down(dec camera theta)\n";
      cerr << "e(dec extrusion)  E(inc extrusion)\n";

      glutMainLoop();
   }
   return 0;
}




